@@ -107,10 +107,7 @@ class GeneralGraph(Graph):
         e1 = self.graph[i, j]
         e2 = self.graph[j, i]
 
-        if (e2 > 4 or e1 > 4) and (e1 != 6 and e2 != 6):
-            return False
-
-        bidirected = e2 == 6 and e1 == 6
+        bidirected = e2 == 1 and e1 == 1
         existing_edge = not bidirected and (e2 != 0 or e1 != 0)
 
         if endpoint1 == "TAIL":
@@ -118,16 +115,16 @@ class GeneralGraph(Graph):
                 return False
             if endpoint2 == "TAIL":
                 if bidirected:
-                    self.graph[j, i] = 5
-                    self.graph[i, j] = 5
+                    self.graph[j, i] = Endpoint.TAIL_AND_ARROW.value
+                    self.graph[i, j] = Endpoint.TAIL_AND_ARROW.value
                 else:
                     self.graph[j, i] = -1
                     self.graph[i, j] = -1
             else:
                 if endpoint2 == "ARROW":
                     if bidirected:
-                        self.graph[j, i] = 7
-                        self.graph[i, j] = 5
+                        self.graph[j, i] = Endpoint.ARROW_AND_ARROW.value
+                        self.graph[i, j] = Endpoint.TAIL_AND_ARROW.value
                     else:
                         self.graph[j, i] = 1
                         self.graph[i, j] = -1
@@ -148,11 +145,17 @@ class GeneralGraph(Graph):
 
                         if e1 == 2 or e2 == 2:
                             return False
-                        self.graph[j, i] = e2 + 6
-                        self.graph[i, j] = e1 + 6
+                        if self.graph[j, i] == Endpoint.ARROW.value:
+                            self.graph[j, i] = Endpoint.ARROW_AND_ARROW.value
+                        else:
+                            self.graph[j, i] = Endpoint.TAIL_AND_ARROW.value
+                        if self.graph[i, j] == Endpoint.ARROW.value:
+                            self.graph[i, j] = Endpoint.ARROW_AND_ARROW.value
+                        else:
+                            self.graph[i, j] = Endpoint.TAIL_AND_ARROW.value
                     else:
-                        self.graph[j, i] = 6
-                        self.graph[i, j] = 6
+                        self.graph[j, i] = Endpoint.ARROW.value
+                        self.graph[i, j] = Endpoint.ARROW.value
                 else:
                     return False
             else:
@@ -234,13 +237,15 @@ class GeneralGraph(Graph):
 
         if endpoint1 == "TAIL":
             if endpoint2 == "TAIL":
-                if (e2 == -1 and e1 == -1) or (e2 == 5 and e1 == 5):
+                if (e2 == -1 and e1 == -1) \
+                        or (e2 == Endpoint.TAIL_AND_ARROW.value and e1 == Endpoint.TAIL_AND_ARROW.value):
                     return True
                 else:
                     return False
             else:
                 if endpoint2 == "ARROW":
-                    if (e1 == -1 and e2 == 1) or (e1 == 5 and e2 == 7):
+                    if (e1 == -1 and e2 == 1) \
+                            or (e1 == Endpoint.TAIL_AND_ARROW.value and e2 == Endpoint.ARROW_AND_ARROW.value):
                         return True
                     else:
                         return False
@@ -255,7 +260,9 @@ class GeneralGraph(Graph):
         else:
             if endpoint1 == "ARROW":
                 if endpoint2 == "ARROW":
-                    if (e1 == 6 and e2 == 6) or (e1 == 5 and e2 == 5) or (e1 == 7 or e2 == 7):
+                    if (e1 == Endpoint.ARROW.value and e2 == Endpoint.ARROW.value) \
+                            or (e1 == Endpoint.TAIL_AND_ARROW.value and e2 == Endpoint.TAIL_AND_ARROW.value) \
+                            or (e1 == Endpoint.ARROW_AND_ARROW.value or e2 == Endpoint.ARROW_AND_ARROW.value):
                         return True
                     else:
                         return False
@@ -346,7 +353,9 @@ class GeneralGraph(Graph):
 
         for i in range(self.num_vars):
 
-            if (self.graph[i, j] == -1 and self.graph[j, i] == 1) or (self.graph[i, j] == 5 and self.graph[j, i] == 7):
+            if (self.graph[i, j] == -1 and self.graph[j, i] == 1) \
+                    or (self.graph[i, j] == Endpoint.TAIL_AND_ARROW.value
+                        and self.graph[j, i] == Endpoint.ARROW_AND_ARROW.value):
                 node2 = self.nodes[i]
                 parents.append(node2)
 
@@ -374,7 +383,9 @@ class GeneralGraph(Graph):
         children = []
 
         for j in range(self.num_vars):
-            if (self.graph[j, i] == 1 and self.graph[i, j] == -1) or (self.graph[j, i] == 7 and self.graph[i, j] == 5):
+            if (self.graph[j, i] == 1 and self.graph[i, j] == -1) \
+                    or (self.graph[j, i] == Endpoint.ARROW_AND_ARROW.value
+                        and self.graph[i, j] == Endpoint.TAIL_AND_ARROW.value):
                 node2 = self.nodes[j]
                 children.append(node2)
 
@@ -388,10 +399,10 @@ class GeneralGraph(Graph):
         indegree = 0
 
         for j in range(self.num_vars):
-            if self.graph[i, j] == 1 or self.graph[i, j] == 6:
+            if self.graph[i, j] == 1:
                 indegree = indegree + 1
             else:
-                if self.graph[i, j] == 7:
+                if self.graph[i, j] == Endpoint.ARROW_AND_ARROW.value:
                     indegree = indegree + 2
 
         return indegree
@@ -404,7 +415,7 @@ class GeneralGraph(Graph):
         outdegree = 0
 
         for j in range(self.num_vars):
-            if self.graph[i, j] == -1 or self.graph[i, j] == 5:
+            if self.graph[i, j] == -1 or self.graph[i, j] == Endpoint.TAIL_AND_ARROW.value:
                 outdegree = outdegree + 1
 
         return outdegree
@@ -418,7 +429,7 @@ class GeneralGraph(Graph):
 
         for j in range(self.num_vars):
 
-            if self.graph[i, j] == 1 or self.graph[i, j] == -1 or self.graph[i, j] == 2 or self.graph[i, j] == 6:
+            if self.graph[i, j] == 1 or self.graph[i, j] == -1 or self.graph[i, j] == 2:
                 degree = degree + 1
             else:
                 if self.graph[i, j] != 0:
@@ -469,7 +480,7 @@ class GeneralGraph(Graph):
 
         for i in range(self.num_vars):
             for j in range(i + 1, self.num_vars):
-                if self.graph[i, j] == 1 or self.graph[i, j] == -1 or self.graph[i, j] == 2 or self.graph[i, j] == 6:
+                if self.graph[i, j] == 1 or self.graph[i, j] == -1 or self.graph[i, j] == 2:
                     edges = edges + 1
                 else:
                     if self.graph[i, j] != 0:
@@ -486,7 +497,7 @@ class GeneralGraph(Graph):
 
         for j in range(self.num_vars):
 
-            if self.graph[j, i] == 1 or self.graph[j, i] == -1 or self.graph[j, i] == 2 or self.graph[j, i] == 6:
+            if self.graph[j, i] == 1 or self.graph[j, i] == -1 or self.graph[j, i] == 2:
                 edges = edges + 1
             else:
                 if self.graph[j, i] != 0:
@@ -520,7 +531,7 @@ class GeneralGraph(Graph):
         i = self.node_map[node1]
         j = self.node_map[node2]
 
-        return self.graph[i, j] == 1 or self.graph[i, j] == 7
+        return self.graph[i, j] == 1 or self.graph[i, j] == Endpoint.ARROW_AND_ARROW.value
 
     # Returns true iff node1 is a parent of node2.
     def is_parent_of(self, node1, node2):
@@ -528,7 +539,7 @@ class GeneralGraph(Graph):
         i = self.node_map[node1]
         j = self.node_map[node2]
 
-        return self.graph[j, i] == 1 or self.graph[j, i] == 7
+        return self.graph[j, i] == 1 or self.graph[j, i] == Endpoint.ARROW_AND_ARROW.value
 
     # Returns true iff node1 is a proper ancestor of node2.
     def is_proper_ancestor_of(self, node1, node2):
@@ -555,10 +566,10 @@ class GeneralGraph(Graph):
         end_1 = self.graph[i, j]
         end_2 = self.graph[j, i]
 
-        if (end_1 > 2 and end_1 != 6) or end_1 == 0:
+        if end_1 == 0:
             return None
 
-        edge = Edge(node1, node2, end_1, end_2)
+        edge = Edge(node1, node2, Endpoint(end_1), Endpoint(end_2))
         return edge
 
     # Returns the directed edge from node1 to node2, if there is one.
@@ -573,7 +584,7 @@ class GeneralGraph(Graph):
         if end_1 > 1 or end_1 == 0 or (end_1 == -1 and end_2 == -1):
             return None
 
-        edge = Edge(node1, node2, end_1, end_2)
+        edge = Edge(node1, node2, Endpoint(end_1), Endpoint(end_2))
         return edge
 
     # Returns the list of edges connected to a particular node. No particular ordering of the edges in the list is guaranteed.
@@ -585,21 +596,24 @@ class GeneralGraph(Graph):
         for j in range(self.num_vars):
 
             node2 = self.nodes[j]
-            if self.graph[j, i] == 1 or self.graph[j, i] == -1 or self.graph[j, i] == 2 or self.graph[j, i] == 6:
+            if self.graph[j, i] == 1 or self.graph[j, i] == -1 or self.graph[j, i] == 2:
                 edges.append(self.get_edge(node, node2))
 
             else:
-                if self.graph[j, i] == 5 and self.graph[i, j] == 7:
-                    edges.append(Edge(node, node2, 1, -1))
-                    edges.append(Edge(node, node2, 6, 6))
+                if self.graph[j, i] == Endpoint.TAIL_AND_ARROW.value \
+                        and self.graph[i, j] == Endpoint.ARROW_AND_ARROW.value:
+                    edges.append(Edge(node, node2, Endpoint.ARROW, Endpoint.TAIL))
+                    edges.append(Edge(node, node2, Endpoint.ARROW, Endpoint.ARROW))
                 else:
-                    if self.graph[j, i] == 7 and self.graph[i, j] == 5:
-                        edges.append(Edge(node, node2, -1, 1))
-                        edges.append(Edge(node, node2, 6, 6))
+                    if self.graph[j, i] == Endpoint.ARROW_AND_ARROW.value \
+                            and self.graph[i, j] == Endpoint.TAIL_AND_ARROW.value:
+                        edges.append(Edge(node, node2, Endpoint.TAIL, Endpoint.ARROW))
+                        edges.append(Edge(node, node2, Endpoint.ARROW, Endpoint.ARROW))
                     else:
-                        if self.graph[j, i] == 5 and self.graph[i, j] == 5:
-                            edges.append(Edge(node, node2, -1, -1))
-                            edges.append(Edge(node, node2, 6, 6))
+                        if self.graph[j, i] == Endpoint.TAIL_AND_ARROW.value \
+                                and self.graph[i, j] == Endpoint.TAIL_AND_ARROW.value:
+                            edges.append(Edge(node, node2, Endpoint.TAIL, Endpoint.TAIL))
+                            edges.append(Edge(node, node2, Endpoint.ARROW, Endpoint.ARROW))
 
         return edges
 
@@ -611,21 +625,24 @@ class GeneralGraph(Graph):
             node = self.nodes[i]
             for j in range(i + 1, self.num_vars):
                 node2 = self.nodes[j]
-                if self.graph[j, i] == 1 or self.graph[j, i] == -1 or self.graph[j, i] == 2 or self.graph[j, i] == 6:
+                if self.graph[j, i] == 1 or self.graph[j, i] == -1 or self.graph[j, i] == 2:
                     edges.append(self.get_edge(node, node2))
 
                 else:
-                    if self.graph[j, i] == 5 and self.graph[i, j] == 7:
-                        edges.append(Edge(node, node2, 1, -1))
-                        edges.append(Edge(node, node2, 6, 6))
+                    if self.graph[j, i] == Endpoint.TAIL_AND_ARROW.value \
+                            and self.graph[i, j] == Endpoint.ARROW_AND_ARROW.value:
+                        edges.append(Edge(node, node2, Endpoint.ARROW, Endpoint.TAIL))
+                        edges.append(Edge(node, node2, Endpoint.ARROW, Endpoint.ARROW))
                     else:
-                        if self.graph[j, i] == 7 and self.graph[i, j] == 5:
-                            edges.append(Edge(node, node2, -1, 1))
-                            edges.append(Edge(node, node2, 6, 6))
+                        if self.graph[j, i] == Endpoint.ARROW_AND_ARROW.value \
+                                and self.graph[i, j] == Endpoint.TAIL_AND_ARROW.value:
+                            edges.append(Edge(node, node2, Endpoint.TAIL, Endpoint.ARROW))
+                            edges.append(Edge(node, node2, Endpoint.ARROW, Endpoint.ARROW))
                         else:
-                            if self.graph[j, i] == 5 and self.graph[i, j] == 5:
-                                edges.append(Edge(node, node2, -1, -1))
-                                edges.append(Edge(node, node2, 6, 6))
+                            if self.graph[j, i] == Endpoint.TAIL_AND_ARROW.value \
+                                    and self.graph[i, j] == Endpoint.TAIL_AND_ARROW.value:
+                                edges.append(Edge(node, node2, Endpoint.TAIL, Endpoint.TAIL))
+                                edges.append(Edge(node, node2, Endpoint.ARROW, Endpoint.ARROW))
 
         return edges
 
@@ -736,13 +753,13 @@ class GeneralGraph(Graph):
 
         if str(endpoint) == "ARROW":
             for j in range(self.num_vars):
-                if self.graph[i, j] == 1 or self.graph[i, j] == 6 or self.graph[i, j] == 7:
+                if self.graph[i, j] == 1 or self.graph[i, j] == Endpoint.ARROW_AND_ARROW.value:
                     node2 = self.nodes[j]
                     nodes.append(node2)
         else:
             if str(endpoint) == "TAIL":
                 for j in range(self.num_vars):
-                    if self.graph[i, j] == -1 or self.graph[i, j] == 5:
+                    if self.graph[i, j] == -1 or self.graph[i, j] == Endpoint.TAIL_AND_ARROW.value:
                         node2 = self.nodes[j]
                         nodes.append(node2)
             else:
@@ -762,13 +779,13 @@ class GeneralGraph(Graph):
 
         if str(endpoint) == "ARROW":
             for j in range(self.num_vars):
-                if self.graph[j, i] == 1 or self.graph[j, i] == 6 or self.graph[j, i] == 7:
+                if self.graph[j, i] == 1 or self.graph[j, i] == Endpoint.ARROW_AND_ARROW.value:
                     node2 = self.nodes[j]
                     nodes.append(node2)
         else:
             if str(endpoint) == "TAIL":
                 for j in range(self.num_vars):
-                    if self.graph[j, i] == -1 or self.graph[j, i] == 5:
+                    if self.graph[j, i] == -1 or self.graph[j, i] == Endpoint.TAIL_AND_ARROW.value:
                         node2 = self.nodes[j]
                         nodes.append(node2)
             else:
@@ -795,32 +812,32 @@ class GeneralGraph(Graph):
         end1 = edge.get_numerical_endpoint1()
         end2 = edge.get_numerical_endpoint2()
 
-        if out_of == 5 and in_to == 5:
-            if end1 == 6:
+        if out_of == Endpoint.TAIL_AND_ARROW.value and in_to == Endpoint.TAIL_AND_ARROW.value:
+            if end1 == Endpoint.ARROW.value:
                 self.graph[j, i] = -1
                 self.graph[i, j] = -1
             else:
                 if end1 == -1:
-                    self.graph[i, j] = 6
-                    self.graph[j, i] = 6
+                    self.graph[i, j] = Endpoint.ARROW.value
+                    self.graph[j, i] = Endpoint.ARROW.value
         else:
-            if out_of == 7 and in_to == 5:
-                if end1 == 6:
+            if out_of == Endpoint.ARROW_AND_ARROW.value and in_to == Endpoint.TAIL_AND_ARROW.value:
+                if end1 == Endpoint.ARROW.value:
                     self.graph[j, i] = 1
                     self.graph[i, j] = -1
                 else:
                     if end1 == -1:
-                        self.graph[j, i] = 6
-                        self.graph[i, j] = 6
+                        self.graph[j, i] = Endpoint.ARROW.value
+                        self.graph[i, j] = Endpoint.ARROW.value
             else:
-                if out_of == 5 and in_to == 7:
-                    if end1 == 6:
+                if out_of == Endpoint.TAIL_AND_ARROW.value and in_to == Endpoint.ARROW_AND_ARROW.value:
+                    if end1 == Endpoint.ARROW.value:
                         self.graph[j, i] = -1
                         self.graph[i, j] = 1
                     else:
                         if end1 == 1:
-                            self.graph[j, i] = 6
-                            self.graph[i, j] = 6
+                            self.graph[j, i] = Endpoint.ARROW.value
+                            self.graph[i, j] = Endpoint.ARROW.value
                 else:
                     if end1 == in_to and end2 == out_of:
                         self.graph[j, i] = 0
@@ -835,11 +852,8 @@ class GeneralGraph(Graph):
         out_of = self.graph[j, i]
         in_to = self.graph[i, j]
 
-        if out_of > 4 and in_to > 4 and out_of != 6 and in_to != 6:
-            pass
-        else:
-            self.graph[j, i] = 0
-            self.graph[i, j] = 0
+        self.graph[j, i] = 0
+        self.graph[i, j] = 0
 
     # Removes all edges connecting node A to node B.  In most cases, this will
     # remove at most one edge, but since multiple edges are permitted in some
