@@ -20,7 +20,7 @@ def local_score_bic(Data, i, PAi, parameters=None):
     score: local BIC score
     '''
 
-    Data = np.asmatrix(Data)
+    Data = np.mat(Data)
     T = Data.shape[0]
     X = Data[:, i]
 
@@ -124,7 +124,7 @@ def local_score_cv_general(Data, Xi, PAi, parameters):
     score: local score
     '''
 
-    Data = np.asmatrix(Data)
+    Data = np.mat(Data)
     PAi = list(PAi)
 
     T = Data.shape[0]
@@ -150,7 +150,7 @@ def local_score_cv_general(Data, Xi, PAi, parameters):
         theta = 1 / (width ** 2)
 
         Kx, _ = kernel(X, X, (theta, 1)) # Gaussian kernel
-        H0 = np.matlib.eye(T) - np.matlib.ones((T, T)) / (T) # for centering of the data in feature space
+        H0 = np.mat(np.eye(T)) - np.mat(np.ones((T, T))) / (T) # for centering of the data in feature space
         Kx = H0 * Kx * H0 # kernel matrix for X
 
         # eig_Kx, eix = eigdec((Kx + Kx.T)/2, np.min([400, math.floor(T/2)]), evals_only=False)   # /2
@@ -160,7 +160,7 @@ def local_score_cv_general(Data, Xi, PAi, parameters):
         # mx = len(IIx)
 
         # set the kernel for PA
-        Kpa = np.matlib.ones((T, T))
+        Kpa = np.mat(np.ones((T, T)))
 
         for m in range(PA.shape[1]):
             G = np.sum((np.multiply(PA[:, m] ,PA[:, m])), axis=1)
@@ -174,7 +174,7 @@ def local_score_cv_general(Data, Xi, PAi, parameters):
             theta = 1 / (width ** 2)
             Kpa = np.multiply(Kpa, kernel(PA[:, m], PA[:, m], (theta, 1))[0])
 
-        H0 = np.matlib.eye(T) - np.matlib.ones((T, T)) / (T)  # for centering of the data in feature space
+        H0 = np.mat(np.eye(T)) - np.mat(np.ones((T, T))) / (T)  # for centering of the data in feature space
         Kpa = H0 * Kpa * H0 # kernel matrix for PA
 
 
@@ -210,15 +210,15 @@ def local_score_cv_general(Data, Xi, PAi, parameters):
                 nv = n0
 
             n1 = T - nv
-            tmp1 = pdinv(Kpa_tr + n1 * var_lambda * np.matlib.eye(n1))
+            tmp1 = pdinv(Kpa_tr + n1 * var_lambda * np.mat(np.eye(n1)))
             tmp2 = tmp1 * Kx_tr * tmp1
-            tmp3 = tmp1 * pdinv(np.matlib.eye(n1) + n1 * var_lambda ** 2 / gamma * tmp2) * tmp1
+            tmp3 = tmp1 * pdinv(np.mat(np.eye(n1)) + n1 * var_lambda ** 2 / gamma * tmp2) * tmp1
             A = (Kx_te + Kpa_tr_te.T * tmp2 * Kpa_tr_te - 2 * Kx_tr_te.T * tmp1 * Kpa_tr_te
                  - n1 * var_lambda ** 2 / gamma * Kx_tr_te.T * tmp3 * Kx_tr_te
                  - n1 * var_lambda ** 2 / gamma * Kpa_tr_te.T * tmp1 * Kx_tr * tmp3 * Kx_tr * tmp1 * Kpa_tr_te
                  + 2 * n1 * var_lambda ** 2 / gamma * Kx_tr_te.T * tmp3 * Kx_tr * tmp1 * Kpa_tr_te) / gamma
 
-            B = n1 * var_lambda ** 2 / gamma * tmp2 + np.matlib.eye(n1)
+            B = n1 * var_lambda ** 2 / gamma * tmp2 + np.mat(np.eye(n1))
             L = np.linalg.cholesky(B)
             C = np.sum(np.log(np.diag(L)))
             #  CV = CV + (nv*nv*log(2*pi) + nv*C + nv*mx*log(gamma) + trace(A))/2;
@@ -238,7 +238,7 @@ def local_score_cv_general(Data, Xi, PAi, parameters):
         theta = 1 / (width ** 2)
 
         Kx, _ = kernel(X, X, (theta, 1))  # Gaussian kernel
-        H0 = np.matlib.eye(T) - np.matlib.ones((T, T)) / (T)  # for centering of the data in feature space
+        H0 = np.mat(np.eye(T)) - np.mat(np.ones((T, T))) / (T)  # for centering of the data in feature space
         Kx = H0 * Kx * H0  # kernel matrix for X
 
         # eig_Kx, eix = eigdec((Kx + Kx.T) / 2, np.min([400, math.floor(T / 2)]), evals_only=False)  # /2
@@ -267,8 +267,8 @@ def local_score_cv_general(Data, Xi, PAi, parameters):
 
             n1 = T - nv
             A = (Kx_te - 1 / (gamma * n1) * Kx_tr_te.T * pdinv(
-                np.matlib.eye(n1) + 1 / (gamma * n1) * Kx_tr) * Kx_tr_te) / gamma
-            B = 1 / (gamma * n1) * Kx_tr + np.matlib.eye(n1)
+                np.mat(np.eye(n1)) + 1 / (gamma * n1) * Kx_tr) * Kx_tr_te) / gamma
+            B = 1 / (gamma * n1) * Kx_tr + np.mat(np.eye(n1))
             L = np.linalg.cholesky(B)
             C = np.sum(np.log(np.diag(L)))
 
@@ -325,11 +325,11 @@ def local_score_cv_multi(Data, Xi, PAi, parameters):
         theta = 1 / (width ** 2 * X.shape[1])  #
 
         Kx, _ = kernel(X, X, (theta, 1))  # Gaussian kernel
-        H0 = np.matlib.eye(T) - np.matlib.ones((T, T)) / (T)  # for centering of the data in feature space
+        H0 = np.mat(np.eye(T)) - np.mat(np.ones((T, T))) / (T)  # for centering of the data in feature space
         Kx = H0 * Kx * H0  # kernel matrix for X
 
         # set the kernel for PA
-        Kpa = np.matlib.ones((T, T))
+        Kpa = np.mat(np.ones((T, T)))
 
         for m in range(len(PAi)):
             PA = Data[:, parameters['dlabel'][PAi[m]]]
@@ -344,7 +344,7 @@ def local_score_cv_multi(Data, Xi, PAi, parameters):
             theta = 1 / (width ** 2 * PA.shape[1])
             Kpa = np.multiply(Kpa, kernel(PA, PA, (theta, 1))[0])
 
-        H0 = np.matlib.eye(T) - np.matlib.ones((T, T)) / (T)  # for centering of the data in feature space
+        H0 = np.mat(np.eye(T)) - np.mat(np.ones((T, T))) / (T)  # for centering of the data in feature space
         Kpa = H0 * Kpa * H0  # kernel matrix for PA
 
         CV = 0
@@ -379,15 +379,15 @@ def local_score_cv_multi(Data, Xi, PAi, parameters):
                 nv = n0
 
             n1 = T - nv
-            tmp1 = pdinv(Kpa_tr + n1 * var_lambda * np.matlib.eye(n1))
+            tmp1 = pdinv(Kpa_tr + n1 * var_lambda * np.mat(np.eye(n1)))
             tmp2 = tmp1 * Kx_tr * tmp1
-            tmp3 = tmp1 * pdinv(np.matlib.eye(n1) + n1 * var_lambda ** 2 / gamma * tmp2) * tmp1
+            tmp3 = tmp1 * pdinv(np.mat(np.eye(n1)) + n1 * var_lambda ** 2 / gamma * tmp2) * tmp1
             A = (Kx_te + Kpa_tr_te.T * tmp2 * Kpa_tr_te - 2 * Kx_tr_te.T * tmp1 * Kpa_tr_te
                  - n1 * var_lambda ** 2 / gamma * Kx_tr_te.T * tmp3 * Kx_tr_te
                  - n1 * var_lambda ** 2 / gamma * Kpa_tr_te.T * tmp1 * Kx_tr * tmp3 * Kx_tr * tmp1 * Kpa_tr_te
                  + 2 * n1 * var_lambda ** 2 / gamma * Kx_tr_te.T * tmp3 * Kx_tr * tmp1 * Kpa_tr_te) / gamma
 
-            B = n1 * var_lambda ** 2 / gamma * tmp2 + np.matlib.eye(n1)
+            B = n1 * var_lambda ** 2 / gamma * tmp2 + np.mat(np.eye(n1))
             L = np.linalg.cholesky(B)
             C = np.sum(np.log(np.diag(L)))
             #  CV = CV + (nv*nv*log(2*pi) + nv*C + nv*mx*log(gamma) + trace(A))/2;
@@ -407,7 +407,7 @@ def local_score_cv_multi(Data, Xi, PAi, parameters):
         theta = 1 / (width ** 2 * X.shape[1])  #
 
         Kx, _ = kernel(X, X, (theta, 1))  # Gaussian kernel
-        H0 = np.matlib.eye(T) - np.matlib.ones((T, T)) / (T)  # for centering of the data in feature space
+        H0 = np.mat(np.eye(T)) - np.mat(np.ones((T, T))) / (T)  # for centering of the data in feature space
         Kx = H0 * Kx * H0  # kernel matrix for X
 
         CV = 0
@@ -432,8 +432,8 @@ def local_score_cv_multi(Data, Xi, PAi, parameters):
 
             n1 = T - nv
             A = (Kx_te - 1 / (gamma * n1) * Kx_tr_te.T * pdinv(
-                np.matlib.eye(n1) + 1 / (gamma * n1) * Kx_tr) * Kx_tr_te) / gamma
-            B = 1 / (gamma * n1) * Kx_tr + np.matlib.eye(n1)
+                np.mat(np.eye(n1)) + 1 / (gamma * n1) * Kx_tr) * Kx_tr_te) / gamma
+            B = 1 / (gamma * n1) * Kx_tr + np.mat(np.eye(n1))
             L = np.linalg.cholesky(B)
             C = np.sum(np.log(np.diag(L)))
 
@@ -477,7 +477,7 @@ def local_score_marginal_general(Data, Xi, PAi, parameters):
     width = np.sqrt(0.5 * np.median(dists[np.where(dists > 0)], axis=1)[0, 0])
     width = width * 2.5  # kernel width
     theta = 1 / (width ** 2)
-    H = np.matlib.eye(T) - np.matlib.ones((T, T)) / T
+    H = np.mat(np.eye(T)) - np.mat(np.ones((T, T))) / T
     Kx, _ = kernel(X, X, (theta, 1))
     Kx = H * Kx * H
 
@@ -490,7 +490,7 @@ def local_score_marginal_general(Data, Xi, PAi, parameters):
     if (len(PAi)):
         PA = Data[:, PAi]
 
-        widthPA = np.matlib.empty((PA.shape[1], 1))
+        widthPA = np.mat(np.empty((PA.shape[1], 1)))
         # set the kernel for PA
         for m in range(PA.shape[1]):
             G = np.sum((np.multiply(PA[:, m], PA[:, m])), axis=1)
@@ -512,8 +512,8 @@ def local_score_marginal_general(Data, Xi, PAi, parameters):
                                          nargout=2)
     else:
         covfunc = np.asarray(['covSum', ['covSEard', 'covNoise']])
-        PA = np.matlib.zeros((T, 1))
-        logtheta0 = np.asmatrix([100, 0, np.log(np.sqrt(0.1))]).T
+        PA = np.mat(np.zeros((T, 1)))
+        logtheta0 = np.mat([100, 0, np.log(np.sqrt(0.1))]).T
         logtheta, fvals, iter = minimize(logtheta0, 'gpr_multi_new', -300, covfunc, PA,
                                               2 * np.sqrt(T) * eix * np.diag(np.sqrt(eig_Kx)) / np.sqrt(eig_Kx[0]))
 
@@ -557,7 +557,7 @@ def local_score_marginal_multi(Data, Xi, PAi, parameters):
     widthX = np.sqrt(0.5 * np.median(dists[np.where(dists > 0)], axis=1)[0, 0])
     widthX = widthX * 2.5  # kernel width
     theta = 1 / (widthX ** 2)
-    H = np.matlib.eye(T) - np.matlib.ones((T, T)) / T
+    H = np.mat(np.eye(T)) - np.mat(np.ones((T, T))) / T
     Kx, _ = kernel(X, X, (theta, 1))
     Kx = H * Kx * H
 
@@ -568,9 +568,9 @@ def local_score_marginal_multi(Data, Xi, PAi, parameters):
     eix = eix[:, IIx]
 
     if (len(PAi)):
-        widthPA_all = np.matlib.empty((1, 0))
+        widthPA_all = np.mat(np.empty((1, 0)))
         # set the kernel for PA
-        PA_all = np.matlib.empty((Data.shape[0], 0))
+        PA_all = np.mat(np.empty((Data.shape[0], 0)))
         for m in range(len(PAi)):
             PA = Data[:, parameters['dlabel'][PAi[m]]]
             PA_all = np.hstack([PA_all, PA])
@@ -581,7 +581,7 @@ def local_score_marginal_multi(Data, Xi, PAi, parameters):
             dists = dists - np.tril(dists)
             dists = np.reshape(dists, (T ** 2, 1))
             widthPA = np.sqrt(0.5 * np.median(dists[np.where(dists > 0)], axis=1)[0, 0])
-            widthPA_all = np.hstack([widthPA_all, widthPA * np.matlib.ones((1, np.size(parameters['dlabel'][PAi[m]])))])
+            widthPA_all = np.hstack([widthPA_all, widthPA * np.mat(np.ones((1, np.size(parameters['dlabel'][PAi[m]]))))])
         widthPA_all = widthPA_all * 2.5  # kernel width
         covfunc = np.asarray(['covSum', ['covSEard', 'covNoise']])
         logtheta0 = np.vstack([np.log(widthPA_all.T), 0, np.log(np.sqrt(0.1))])
@@ -593,8 +593,8 @@ def local_score_marginal_multi(Data, Xi, PAi, parameters):
                                          nargout=2)
     else:
         covfunc = np.asarray(['covSum', ['covSEard', 'covNoise']])
-        PA = np.matlib.zeros((T, 1))
-        logtheta0 = np.asmatrix([100, 0, np.log(np.sqrt(0.1))]).T
+        PA = np.mat(np.zeros((T, 1)))
+        logtheta0 = np.mat([100, 0, np.log(np.sqrt(0.1))]).T
         logtheta, fvals, iter = minimize(logtheta0, 'gpr_multi_new', -300, covfunc, PA,
                                               2 * np.sqrt(T) * eix * np.diag(np.sqrt(eig_Kx)) / np.sqrt(eig_Kx[0]))
 
