@@ -448,7 +448,7 @@ class GraphUtils:
 
     def directed(self, e: Edge):
         return (e.get_endpoint1() == Endpoint.TAIL and e.get_endpoint2() == Endpoint.ARROW) \
-            or (e.get_endpoint1() == Endpoint.HEAD and e.get_endpoint2() == Endpoint.TAIL)
+            or (e.get_endpoint1() == Endpoint.ARROW and e.get_endpoint2() == Endpoint.TAIL)
 
     def bi_directed(self, e: Edge):
         return e.get_endpoint1() == Endpoint.ARROW and e.get_endpoint2() == Endpoint.ARROW
