@@ -5,6 +5,8 @@ from pytrad.utils.KCI.KCI import KCI_UInd, KCI_CInd
 from pytrad.utils.KCI.GaussianKernel import GaussianKernel
 from pytrad.utils.KCI.PolynomialKernel import PolynomialKernel
 from pytrad.utils.KCI.LinearKernel import LinearKernel
+from pytrad.utils.PCUtils import Helper
+
 
 
 def kci(data, X, Y, condition_set=None, kernelX='Gaussian', kernelY='Gaussian', kernelZ='Gaussian',
@@ -65,7 +67,7 @@ def kci_ci(X, Y, Z, kernelX='Gaussian', kernelY='Gaussian', kernelZ='Gaussian',
     pvalue, _ = kci_cind.compute_pvalue(X, Y, Z)
     return pvalue
 
-def mv_fisherz(mvdata, X, Y, condition_set):
+def mv_fisherz(mvdata, X, Y, condition_set, sample_size = None):
     '''
     Perform an independence test using Fisher-Z's test for data with missing values
 
@@ -89,6 +91,61 @@ def mv_fisherz(mvdata, X, Y, condition_set):
     return p
 
 
+def mc_fisherz(mdata, skel, prt_m, X, Y, condition_set, sample_size):
+    """Perform an independent test using Fisher-Z's test with test-wise deletion and missingness correction
+    If it is not the case which requires a correction, then call function mvfisherZ(...)
+    :param prt_m: dictionary, with elements:
+        - m: missingness indicators which are not MCAR
+        - prt: parents of the missingness indicators
+    """
+
+    ## Check whether whether there is at least one common child of X and Y
+    if not Helper.cond_perm_c(X, Y, condition_set, prt_m, skel):
+        return mv_fisherz(mdata, X, Y, condition_set, sample_size)
+
+    ## *********** Step 1 ***********
+    # Learning generaive model for {X, Y, S} to impute X, Y, and S
+
+    ## Get parents the {xyS} missingness indicators with parents: prt_m
+    # W is the variable which can be used for missingness correction
+    W_indx_ = Helper.get_prt_mvars(var=list((X, Y) + condition_set), prt_m=prt_m)
+
+    if len(W_indx_) == 0:  # When there is no variable can be used for correction
+        return mv_fisherz(mdata, X, Y, condition_set, sample_size)
+
+    ## Get the parents of W missingness indicators
+    W_indx = Helper.get_prt_mw(W_indx_, prt_m)
+
+    ## Prepare the W for regression
+    # Since the XYS will be regressed on W,
+    # W will not contain any of XYS
+    var = list((X, Y) + condition_set)
+    W_indx = list(set(W_indx) - set(var))
+
+    if len(W_indx) == 0:  # When there is no variable can be used for correction
+        return mv_fisherz(mdata, X, Y, condition_set, sample_size)
+
+    ## Learn regression models with test-wise deleted data
+    involve_vars = var + W_indx
+    tdel_data = Helper.test_wise_deletion(mdata[:, involve_vars])
+    effective_sz = len(tdel_data[:, 0])
+    regMs, rss = Helper.learn_regression_model(tdel_data, num_model=len(var))
+
+    ## *********** Step 2 ***********
+    # Get the data of the predictors, Ws
+    Ws = Helper.get_predictor_ws(mdata[:, involve_vars], num_test_var=len(var), effective_sz=effective_sz)
+
+    ## *********** Step 3 ***********
+    # Generate the virtual data follows the full data distribution P(X, Y, S)
+    data_vir = Helper.gen_vir_data(regMs, rss, Ws, len(var), effective_sz)
+
+    if len(var) > 2:
+        cond_set_bgn_0 = np.arange(2, len(var))
+    else:
+        cond_set_bgn_0 = []
+
+    return mv_fisherz(data_vir, 0, 1, tuple(cond_set_bgn_0), effective_sz)
+
 def fisherz(data, X, Y, condition_set, correlation_matrix=None):
     '''
     Perform an independence test using Fisher-Z's test
